
package com.providence;

import java.rmi.RemoteException;
import javax.ejb.*;

import com.providence.plan.*;


/**
 * This stateless session bean implements a test web service for the 
 * Providence Insurance Company which provides information about medical 
 * and dental plans. 
 * The service endpoint interface for this EJB is 
 * com.providence.plan.ProvidenceWebPort which is generated from the
 * Providence.wsdl description.
 */

public class ProvidenceBean implements SessionBean {

    private static final String MEDICAL_PLAN = "MEDICAL_PLAN";
    private static final String DENTAL_PLAN = "DENTAL_PLAN";
    private PlanInfo[] medicalPlans;
    private PlanInfo[] dentalPlans;

    public ProvidenceBean() {
	medicalPlans = new PlanInfo[1];
	dentalPlans = new PlanInfo[1];

	medicalPlans[0] = new PlanInfo(9.0, 150.0, "ProvidenceMedical1", "Providence Medical Plan 1", MEDICAL_PLAN, 200.0);

	dentalPlans[0] = new PlanInfo(9.0, 150.0, "ProvidenceDental1", "Providence Dental Plan 1", DENTAL_PLAN, 250.0);
    }

    public double getCost(String planId, int coverage, int age, 
	    boolean isSmoker) throws java.rmi.RemoteException {

	for ( int i=0; i<medicalPlans.length; i++ ) {
	    if ( planId.equals(medicalPlans[i].getPlanId()) ) {
		double cost = medicalPlans[i].getCoverageFactor() * (coverage+1)
			    + medicalPlans[i].getAgeFactor() * age;
		if ( isSmoker ) {
		    cost += medicalPlans[i].getSmokerCost();
		}
		return cost;
	    }
	}
	for ( int i=0; i<dentalPlans.length; i++ ) {
	    if ( planId.equals(dentalPlans[i].getPlanId()) ) {
		double cost = dentalPlans[i].getCoverageFactor() * coverage
			    + dentalPlans[i].getAgeFactor() * age;
		if ( isSmoker ) {
		    cost += dentalPlans[i].getSmokerCost();
		}
		return cost;
	    }
	}
	throw new RemoteException("Unknown plan id");
    }

    public DoctorInfo[] getDoctorInfo(String planId, DoctorInfo doctorQuery) 
	    throws java.rmi.RemoteException {

	throw new RemoteException("Not implemented");
    }

    public PlanInfo[] getPlanInfo(String planId, String planType, 
	    DoctorInfo doctorQuery) throws java.rmi.RemoteException {

	System.err.println("In getPlanInfo, planId = " + planId 
		+ ", planType=" + planType + ", DoctorInfo = " + doctorQuery);

	if ( planId != null ) {
	    for ( int i=0; i<medicalPlans.length; i++ ) {
		if ( planId.equals(medicalPlans[i].getPlanId()) ) {
		    PlanInfo[] plans = new PlanInfo[1];
		    plans[0] = medicalPlans[i];
		    return plans;
		}
	    }
	    for ( int i=0; i<dentalPlans.length; i++ ) {
		if ( planId.equals(dentalPlans[i].getPlanId()) ) {
		    PlanInfo[] plans = new PlanInfo[1];
		    plans[0] = dentalPlans[i];
		    return plans;
		}
	    }
	    throw new RemoteException("Bad plan id");
	}
	else if ( planType != null && doctorQuery == null ) {
	    if ( planType.equals(MEDICAL_PLAN) )
		return medicalPlans;
	    else if ( planType.equals(DENTAL_PLAN) )
		return dentalPlans;
	    else
		throw new RemoteException("Bad plan type");
	} 
	else {
	    throw new RemoteException("Unimplemented plan query");
	}
    }    


    /** Lifecycle methods **/

    public void setSessionContext(SessionContext sessionContext) {}

    public void ejbCreate() {}

    public void ejbRemove() {}

    public void ejbActivate() {}

    public void ejbPassivate() {}
}

